@@ -1,4 +1,3 @@
-# flake8: noqa
 # Copyright 2020 The HuggingFace Datasets Authors and the TensorFlow Datasets Authors.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -13,32 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# Lint as: python3
-# pylint: enable=line-too-long
-# pylint: disable=g-import-not-at-top,g-bad-import-order,wrong-import-position
-
-__version__ = "2.10.1"
-
-import platform
-
-import pyarrow
-from packaging import version
-
-
-if version.parse(platform.python_version()) < version.parse("3.7"):
-    raise ImportWarning(
-        "To use `datasets`, Python>=3.7 is required, and the current version of Python doesn't match this condition."
-    )
-
-if version.parse(pyarrow.__version__).major < 6:
-    raise ImportWarning(
-        "To use `datasets`, the module `pyarrow>=6.0.0` is required, and the current version of `pyarrow` doesn't match this condition.\n"
-        "If you are running this in a Google Colab, you should probably just restart the runtime to use the right version of `pyarrow`."
-    )
-
-del platform
-del pyarrow
-del version
+__version__ = "2.21.0"
 
 from .arrow_dataset import Dataset
 from .arrow_reader import ReadInstruction
@@ -52,6 +26,7 @@ from .info import DatasetInfo, MetricInfo
 from .inspect import (
     get_dataset_config_info,
     get_dataset_config_names,
+    get_dataset_default_config_name,
     get_dataset_infos,
     get_dataset_split_names,
     inspect_dataset,
@@ -78,10 +53,15 @@ from .utils import *
 from .utils import logging
 
 
-# deprecated modules
-from datasets import arrow_dataset as _arrow_dataset  # isort:skip
-from datasets import utils as _utils  # isort:skip
-from datasets.utils import download_manager as _deprecated_download_manager  # isort:skip
+# isort: split
+
+# Deprecated modules
+from . import arrow_dataset as _arrow_dataset
+from . import utils as _utils
+from .exceptions import ExpectedMoreDownloadedFiles, ExpectedMoreSplits, UnexpectedDownloadedFile, UnexpectedSplits
+from .utils import download_manager as _deprecated_download_manager
+from .utils import info_utils as _deprecated_info_utils
+
 
 _arrow_dataset.concatenate_datasets = concatenate_datasets
 _utils.DownloadConfig = DownloadConfig
@@ -90,5 +70,10 @@ _utils.DownloadMode = DownloadMode
 _deprecated_download_manager.DownloadConfig = DownloadConfig
 _deprecated_download_manager.DownloadMode = DownloadMode
 _deprecated_download_manager.DownloadManager = DownloadManager
+_deprecated_info_utils.ExpectedMoreDownloadedFiles = ExpectedMoreDownloadedFiles
+_deprecated_info_utils.ExpectedMoreSplits = ExpectedMoreSplits
+_deprecated_info_utils.UnexpectedDownloadedFile = UnexpectedDownloadedFile
+_deprecated_info_utils.UnexpectedSplits = UnexpectedSplits
 
 del _arrow_dataset, _utils, _deprecated_download_manager
+del _deprecated_info_utils, ExpectedMoreDownloadedFiles, ExpectedMoreSplits, UnexpectedDownloadedFile, UnexpectedSplits
