@@ -1,15 +1,15 @@
+import logging
 import os
 from argparse import ArgumentParser
 from pathlib import Path
 from shutil import copyfile, rmtree
-from typing import Generator
+from typing import Generator, Optional
 
 import datasets.config
 from datasets.builder import DatasetBuilder
 from datasets.commands import BaseDatasetsCLICommand
 from datasets.download.download_manager import DownloadMode
 from datasets.load import dataset_module_factory, import_main_class
-from datasets.utils.filelock import logger as fl_logger
 from datasets.utils.info_utils import VerificationMode
 from datasets.utils.logging import ERROR, get_logger
 
@@ -28,6 +28,8 @@ def _test_command_factory(args):
         args.ignore_verifications,
         args.force_redownload,
         args.clear_cache,
+        args.num_proc,
+        args.trust_remote_code,
     )
 
 
@@ -65,6 +67,10 @@ class TestCommand(BaseDatasetsCLICommand):
             action="store_true",
             help="Remove downloaded files and cached datasets after each config test",
         )
+        test_parser.add_argument("--num_proc", type=int, default=None, help="Number of processes")
+        test_parser.add_argument(
+            "--trust_remote_code", action="store_true", help="whether to trust the code execution of the load script"
+        )
         # aliases
         test_parser.add_argument("--save_infos", action="store_true", help="alias to save_info")
         test_parser.add_argument("dataset", type=str, help="Name of the dataset to download")
@@ -81,6 +87,8 @@ class TestCommand(BaseDatasetsCLICommand):
         ignore_verifications: bool,
         force_redownload: bool,
         clear_cache: bool,
+        num_proc: int,
+        trust_remote_code: Optional[bool],
     ):
         self._dataset = dataset
         self._name = name
@@ -91,6 +99,8 @@ class TestCommand(BaseDatasetsCLICommand):
         self._ignore_verifications = ignore_verifications
         self._force_redownload = force_redownload
         self._clear_cache = clear_cache
+        self._num_proc = num_proc
+        self._trust_remote_code = trust_remote_code
         if clear_cache and not cache_dir:
             print(
                 "When --clear_cache is used, specifying a cache directory is mandatory.\n"
@@ -102,12 +112,12 @@ class TestCommand(BaseDatasetsCLICommand):
             self._ignore_verifications = True
 
     def run(self):
-        fl_logger().setLevel(ERROR)
+        logging.getLogger("filelock").setLevel(ERROR)
         if self._name is not None and self._all_configs:
             print("Both parameters `config` and `all_configs` can't be used at once.")
             exit(1)
         path, config_name = self._dataset, self._name
-        module = dataset_module_factory(path)
+        module = dataset_module_factory(path, trust_remote_code=self._trust_remote_code)
         builder_cls = import_main_class(module.module_path)
         n_builders = len(builder_cls.BUILDER_CONFIGS) if self._all_configs and builder_cls.BUILDER_CONFIGS else 1
 
@@ -140,7 +150,9 @@ class TestCommand(BaseDatasetsCLICommand):
 
         for j, builder in enumerate(get_builders()):
             print(f"Testing builder '{builder.config.name}' ({j + 1}/{n_builders})")
-            builder._record_infos = True
+            builder._record_infos = os.path.exists(
+                os.path.join(builder.get_imported_module_dir(), datasets.config.DATASETDICT_INFOS_FILENAME)
+            )  # record checksums only if we need to update a (deprecated) dataset_infos.json
             builder.download_and_prepare(
                 download_mode=DownloadMode.REUSE_CACHE_IF_EXISTS
                 if not self._force_redownload
@@ -149,6 +161,7 @@ class TestCommand(BaseDatasetsCLICommand):
                 if self._ignore_verifications
                 else VerificationMode.ALL_CHECKS,
                 try_from_hf_gcs=False,
+                num_proc=self._num_proc,
             )
             builder.as_dataset()
             if self._save_infos:
@@ -160,7 +173,9 @@ class TestCommand(BaseDatasetsCLICommand):
             # Let's move it to the original directory of the dataset script, to allow the user to
             # upload them on S3 at the same time afterwards.
             if self._save_infos:
-                dataset_readme_path = os.path.join(builder_cls.get_imported_module_dir(), "README.md")
+                dataset_readme_path = os.path.join(
+                    builder_cls.get_imported_module_dir(), datasets.config.REPOCARD_FILENAME
+                )
                 name = Path(path).name + ".py"
                 combined_path = os.path.join(path, name)
                 if os.path.isfile(path):
@@ -175,7 +190,7 @@ class TestCommand(BaseDatasetsCLICommand):
 
                 # Move dataset_info back to the user
                 if dataset_dir is not None:
-                    user_dataset_readme_path = os.path.join(dataset_dir, "README.md")
+                    user_dataset_readme_path = os.path.join(dataset_dir, datasets.config.REPOCARD_FILENAME)
                     copyfile(dataset_readme_path, user_dataset_readme_path)
                     print(f"Dataset card saved at {user_dataset_readme_path}")
 
