@@ -2,7 +2,7 @@ import collections
 import itertools
 import os
 from dataclasses import dataclass
-from typing import List, Optional, Tuple
+from typing import List, Optional, Tuple, Type
 
 import pandas as pd
 import pyarrow as pa
@@ -16,19 +16,6 @@ from datasets.tasks.base import TaskTemplate
 logger = datasets.utils.logging.get_logger(__name__)
 
 
-if datasets.config.PYARROW_VERSION.major >= 7:
-
-    def pa_table_to_pylist(table):
-        return table.to_pylist()
-
-else:
-
-    def pa_table_to_pylist(table):
-        keys = table.column_names
-        values = table.to_pydict().values()
-        return [{k: v for k, v in zip(keys, row_values)} for row_values in zip(*values)]
-
-
 def count_path_segments(path):
     return path.replace("\\", "/").count("/")
 
@@ -41,6 +28,9 @@ class FolderBasedBuilderConfig(datasets.BuilderConfig):
     drop_labels: bool = None
     drop_metadata: bool = None
 
+    def __post_init__(self):
+        super().__post_init__()
+
 
 class FolderBasedBuilder(datasets.GeneratorBasedBuilder):
     """
@@ -48,7 +38,7 @@ class FolderBasedBuilder(datasets.GeneratorBasedBuilder):
 
 
     Abstract class attributes to be overridden by a child class:
-        BASE_FEATURE: feature object to decode data (i.e. datasets.Image(), datasets.Audio(), ...)
+        BASE_FEATURE: feature object to decode data (i.e. datasets.Image, datasets.Audio, ...)
         BASE_COLUMN_NAME: string key name of a base feature (i.e. "image", "audio", ...)
         BUILDER_CONFIG_CLASS: builder config inherited from `folder_based_builder.FolderBasedBuilderConfig`
         EXTENSIONS: list of allowed extensions (only files with these extensions and METADATA_FILENAME files
@@ -56,7 +46,7 @@ class FolderBasedBuilder(datasets.GeneratorBasedBuilder):
         CLASSIFICATION_TASK: classification task to use if labels are obtained from the folder structure
     """
 
-    BASE_FEATURE: FeatureType
+    BASE_FEATURE: Type[FeatureType]
     BASE_COLUMN_NAME: str
     BUILDER_CONFIG_CLASS: FolderBasedBuilderConfig
     EXTENSIONS: List[str]
@@ -70,7 +60,7 @@ class FolderBasedBuilder(datasets.GeneratorBasedBuilder):
     def _split_generators(self, dl_manager):
         if not self.config.data_files:
             raise ValueError(f"At least one data file must be specified, but got data_files={self.config.data_files}")
-
+        dl_manager.download_config.extract_on_the_fly = True
         # Do an early pass if:
         # * `drop_labels` is None (default) or False, to infer the class labels
         # * `drop_metadata` is None (default) or False, to find the metadata files
@@ -159,7 +149,7 @@ class FolderBasedBuilder(datasets.GeneratorBasedBuilder):
                 datasets.SplitGenerator(
                     name=split_name,
                     gen_kwargs={
-                        "files": [(file, downloaded_file) for file, downloaded_file in zip(files, downloaded_files)]
+                        "files": list(zip(files, downloaded_files))
                         + [(None, dl_manager.iter_files(downloaded_dir)) for downloaded_dir in downloaded_dirs],
                         "metadata_files": metadata_files,
                         "split_name": split_name,
@@ -176,16 +166,16 @@ class FolderBasedBuilder(datasets.GeneratorBasedBuilder):
             features_per_metadata_file: List[Tuple[str, datasets.Features]] = []
 
             # Check that all metadata files share the same format
-            metadata_ext = set(
-                os.path.splitext(downloaded_metadata_file)[1][1:]
-                for _, downloaded_metadata_file in itertools.chain.from_iterable(metadata_files.values())
-            )
+            metadata_ext = {
+                os.path.splitext(original_metadata_file)[-1]
+                for original_metadata_file, _ in itertools.chain.from_iterable(metadata_files.values())
+            }
             if len(metadata_ext) > 1:
                 raise ValueError(f"Found metadata files with different extensions: {list(metadata_ext)}")
             metadata_ext = metadata_ext.pop()
 
             for _, downloaded_metadata_file in itertools.chain.from_iterable(metadata_files.values()):
-                pa_metadata_table = self._read_metadata(downloaded_metadata_file)
+                pa_metadata_table = self._read_metadata(downloaded_metadata_file, metadata_ext=metadata_ext)
                 features_per_metadata_file.append(
                     (downloaded_metadata_file, datasets.Features.from_arrow_schema(pa_metadata_table.schema))
                 )
@@ -209,13 +199,13 @@ class FolderBasedBuilder(datasets.GeneratorBasedBuilder):
             if add_labels:
                 self.info.features = datasets.Features(
                     {
-                        self.BASE_COLUMN_NAME: self.BASE_FEATURE,
+                        self.BASE_COLUMN_NAME: self.BASE_FEATURE(),
                         "label": datasets.ClassLabel(names=sorted(labels)),
                     }
                 )
                 self.info.task_templates = [self.CLASSIFICATION_TASK.align_with_features(self.info.features)]
             else:
-                self.info.features = datasets.Features({self.BASE_COLUMN_NAME: self.BASE_FEATURE})
+                self.info.features = datasets.Features({self.BASE_COLUMN_NAME: self.BASE_FEATURE()})
 
             if add_metadata:
                 # Warn if there are duplicated keys in metadata compared to the existing features
@@ -249,9 +239,8 @@ class FolderBasedBuilder(datasets.GeneratorBasedBuilder):
                 archives.append(data_file)
         return files, archives
 
-    def _read_metadata(self, metadata_file):
-        metadata_file_ext = os.path.splitext(metadata_file)[1][1:]
-        if metadata_file_ext == "csv":
+    def _read_metadata(self, metadata_file, metadata_ext: str = ""):
+        if metadata_ext == ".csv":
             # Use `pd.read_csv` (although slower) instead of `pyarrow.csv.read_csv` for reading CSV files for consistency with the CSV packaged module
             return pa.Table.from_pandas(pd.read_csv(metadata_file))
         else:
@@ -268,11 +257,11 @@ class FolderBasedBuilder(datasets.GeneratorBasedBuilder):
         metadata_dict = None
         downloaded_metadata_file = None
 
+        metadata_ext = ""
         if split_metadata_files:
-            metadata_ext = set(
-                os.path.splitext(downloaded_metadata_file)[1][1:]
-                for _, downloaded_metadata_file in split_metadata_files
-            )
+            metadata_ext = {
+                os.path.splitext(original_metadata_file)[-1] for original_metadata_file, _ in split_metadata_files
+            }
             metadata_ext = metadata_ext.pop()
 
         file_idx = 0
@@ -303,19 +292,21 @@ class FolderBasedBuilder(datasets.GeneratorBasedBuilder):
                                 _, metadata_file, downloaded_metadata_file = min(
                                     metadata_file_candidates, key=lambda x: count_path_segments(x[0])
                                 )
-                                pa_metadata_table = self._read_metadata(downloaded_metadata_file)
+                                pa_metadata_table = self._read_metadata(
+                                    downloaded_metadata_file, metadata_ext=metadata_ext
+                                )
                                 pa_file_name_array = pa_metadata_table["file_name"]
                                 pa_metadata_table = pa_metadata_table.drop(["file_name"])
                                 metadata_dir = os.path.dirname(metadata_file)
                                 metadata_dict = {
                                     os.path.normpath(file_name).replace("\\", "/"): sample_metadata
                                     for file_name, sample_metadata in zip(
-                                        pa_file_name_array.to_pylist(), pa_table_to_pylist(pa_metadata_table)
+                                        pa_file_name_array.to_pylist(), pa_metadata_table.to_pylist()
                                     )
                                 }
                             else:
                                 raise ValueError(
-                                    f"One or several metadata.{metadata_ext} were found, but not in the same directory or in a parent directory of {downloaded_file_or_dir}."
+                                    f"One or several metadata{metadata_ext} were found, but not in the same directory or in a parent directory of {downloaded_file_or_dir}."
                                 )
                         if metadata_dir is not None and downloaded_metadata_file is not None:
                             file_relpath = os.path.relpath(original_file, metadata_dir)
@@ -327,7 +318,7 @@ class FolderBasedBuilder(datasets.GeneratorBasedBuilder):
                             sample_metadata = metadata_dict[file_relpath]
                         else:
                             raise ValueError(
-                                f"One or several metadata.{metadata_ext} were found, but not in the same directory or in a parent directory of {downloaded_file_or_dir}."
+                                f"One or several metadata{metadata_ext} were found, but not in the same directory or in a parent directory of {downloaded_file_or_dir}."
                             )
                     else:
                         sample_metadata = {}
@@ -335,12 +326,15 @@ class FolderBasedBuilder(datasets.GeneratorBasedBuilder):
                         sample_label = {"label": os.path.basename(os.path.dirname(original_file))}
                     else:
                         sample_label = {}
-                    yield file_idx, {
-                        **sample_empty_metadata,
-                        self.BASE_COLUMN_NAME: downloaded_file_or_dir,
-                        **sample_metadata,
-                        **sample_label,
-                    }
+                    yield (
+                        file_idx,
+                        {
+                            **sample_empty_metadata,
+                            self.BASE_COLUMN_NAME: downloaded_file_or_dir,
+                            **sample_metadata,
+                            **sample_label,
+                        },
+                    )
                     file_idx += 1
             else:
                 for downloaded_dir_file in downloaded_file_or_dir:
@@ -369,19 +363,21 @@ class FolderBasedBuilder(datasets.GeneratorBasedBuilder):
                                     _, metadata_file, downloaded_metadata_file = min(
                                         metadata_file_candidates, key=lambda x: count_path_segments(x[0])
                                     )
-                                    pa_metadata_table = self._read_metadata(downloaded_metadata_file)
+                                    pa_metadata_table = self._read_metadata(
+                                        downloaded_metadata_file, metadata_ext=metadata_ext
+                                    )
                                     pa_file_name_array = pa_metadata_table["file_name"]
                                     pa_metadata_table = pa_metadata_table.drop(["file_name"])
                                     metadata_dir = os.path.dirname(downloaded_metadata_file)
                                     metadata_dict = {
                                         os.path.normpath(file_name).replace("\\", "/"): sample_metadata
                                         for file_name, sample_metadata in zip(
-                                            pa_file_name_array.to_pylist(), pa_table_to_pylist(pa_metadata_table)
+                                            pa_file_name_array.to_pylist(), pa_metadata_table.to_pylist()
                                         )
                                     }
                                 else:
                                     raise ValueError(
-                                        f"One or several metadata.{metadata_ext} were found, but not in the same directory or in a parent directory of {downloaded_dir_file}."
+                                        f"One or several metadata{metadata_ext} were found, but not in the same directory or in a parent directory of {downloaded_dir_file}."
                                     )
                             if metadata_dir is not None and downloaded_metadata_file is not None:
                                 downloaded_dir_file_relpath = os.path.relpath(downloaded_dir_file, metadata_dir)
@@ -393,7 +389,7 @@ class FolderBasedBuilder(datasets.GeneratorBasedBuilder):
                                 sample_metadata = metadata_dict[downloaded_dir_file_relpath]
                             else:
                                 raise ValueError(
-                                    f"One or several metadata.{metadata_ext} were found, but not in the same directory or in a parent directory of {downloaded_dir_file}."
+                                    f"One or several metadata{metadata_ext} were found, but not in the same directory or in a parent directory of {downloaded_dir_file}."
                                 )
                         else:
                             sample_metadata = {}
@@ -401,10 +397,13 @@ class FolderBasedBuilder(datasets.GeneratorBasedBuilder):
                             sample_label = {"label": os.path.basename(os.path.dirname(downloaded_dir_file))}
                         else:
                             sample_label = {}
-                        yield file_idx, {
-                            **sample_empty_metadata,
-                            self.BASE_COLUMN_NAME: downloaded_dir_file,
-                            **sample_metadata,
-                            **sample_label,
-                        }
+                        yield (
+                            file_idx,
+                            {
+                                **sample_empty_metadata,
+                                self.BASE_COLUMN_NAME: downloaded_dir_file,
+                                **sample_metadata,
+                                **sample_label,
+                            },
+                        )
                         file_idx += 1
