@@ -1,4 +1,4 @@
-from dataclasses import dataclass
+from dataclasses import dataclass, field
 from typing import ClassVar, Dict
 
 from ..features import Features, Sequence, Value
@@ -8,7 +8,7 @@ from .base import TaskTemplate
 @dataclass(frozen=True)
 class QuestionAnsweringExtractive(TaskTemplate):
     # `task` is not a ClassVar since we want it to be part of the `asdict` output for JSON serialization
-    task: str = "question-answering-extractive"
+    task: str = field(default="question-answering-extractive", metadata={"include_in_asdict_even_if_is_default": True})
     input_schema: ClassVar[Features] = Features({"question": Value("string"), "context": Value("string")})
     label_schema: ClassVar[Features] = Features(
         {
