@@ -10,7 +10,7 @@ import pytest
 from datasets.features import Features, Sequence, Value
 from datasets.metric import Metric, MetricInfo
 
-from .utils import require_tf, require_torch
+from .utils import require_numpy1_on_windows, require_tf, require_torch
 
 
 class DummyMetric(Metric):
@@ -130,6 +130,7 @@ def metric_add_and_compute(arg):
         properly_del_metric(metric)
 
 
+@pytest.mark.filterwarnings("ignore:Metric is deprecated:FutureWarning")
 class TestMetric(TestCase):
     def test_dummy_metric(self):
         preds, refs = DummyMetric.predictions_and_references()
@@ -432,6 +433,7 @@ class TestMetric(TestCase):
         self.assertDictEqual(expected_results, metric.compute())
         del metric
 
+    @require_numpy1_on_windows
     @require_torch
     def test_input_torch(self):
         import torch
